import logging
import sys

from concurrent_log_handler import ConcurrentRotatingFileHandler

# 设置日志的名称
LOGGER_NAME = __name__
# 指定日志的等级必须在LOG_LEVEL变量里
LOG_LEVEL = ['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG']


class MyLogger:
    # 创建一个日志对象
    def __init__(self, logger_name=LOGGER_NAME,
                 log_level: LOG_LEVEL = None,
                 file_name=None,
                 file_size=512 * 1024,
                 log_format="%(asctime)s - %(name)s - %(levelname)s - %(process)d - %(processName)s -%(thread)d -%("
                            "threadName)s - %(message)s"):

        self.log_level = log_level
        # 创建一个格式器
        self.formatter = logging.Formatter(log_format)
        # 指定log文件
        self.file_name = file_name
        # 指定log文件大小
        self.file_size = file_size
        # 指定logger名称
        self.logger_name = LOGGER_NAME

        self.logger = logging.getLogger(logger_name)

        # 设置日志等级
        try:
            self.logger.setLevel(logging.DEBUG if not self.log_level else self.log_level)
        except Exception:
            self.logger.error('Log等级必须属于LOG_LEVEL变量')
            raise ValueError('Log等级必须属于LOG_LEVEL变量')

        # 为日志对象添加流的处理器handler
        self.logger.addHandler(self.get_console_handler())

        # 为日志对象添加文件处理器handler
        if file_name:
            self.logger.addHandler(self.get_file_handler())

        self.propagate = False

    def get_logger(self, module_name):
        return logging.getLogger(self.logger_name).getChild(module_name)

    def get_console_handler(self):
        # 创建一个流处理器handler
        ch = logging.StreamHandler(sys.stdout)
        # 创建一个日志格式器formatter并将其添加到处理器handler
        ch.setFormatter(self.formatter)
        self.logger.handlers.clear()
        return ch

    def get_file_handler(self):
        # 使用进程安全的方式创建
        # 指定文件写入方式为append，文件大小为512k
        fh = ConcurrentRotatingFileHandler(self.file_name, "a", self.file_size, encoding="utf-8")
        fh.setFormatter(self.formatter)
        fh.setLevel("DEBUG")
        return fh

    def info(self, msg, extra=None):
        self.logger.info(msg, extra=extra)

    def error(self, msg, extra=None):
        self.logger.error(msg, extra=extra)

    def debug(self, msg, extra=None):
        self.logger.debug(msg, extra=extra)

    def warning(self, msg, extra=None):
        self.logger.warning(msg, extra=extra)



from functools import wraps


class Test(object):
    # enabled决定使用标签挑选测试用例是否启用
    # 默认启用，如为False，则不启用
    # tag 为用户自定义标签
    def __init__(self, tag=None, enabled=True):
        self.enabled = enabled
        self.tag = tag

    def __call__(self, func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)
        # 给原测试函数添加属性，以方便测试框架判断当前测试函数是否为测试用例
        setattr(wrapper, "__test_tag__", self.tag)
        setattr(wrapper, "__test_case_type__", "__TestCase__")
        setattr(wrapper, "__test_case_enabled__", self.enabled)
        return wrapper







# -*- coding: utf-8 -*-
import functools
import os
import platform
import shutil
import time
import traceback
from collections import OrderedDict

from ztDemo.ztDemo.common.customize_error import RunTimeTooLong
from ztDemo.ztDemo.common.html_report import generate_html_report
from ztDemo.ztDemo.common.my_logger import MyLogger
from ztDemo.ztDemo.common.test_filter import TestFilter
from ztDemo.ztDemo.common.test_finder import DiscoverTestCases
from ztDemo.ztDemo.common.user_options import parse_options
from ztDemo.ztDemo.settings.global_config import init, set_config, get_config
from multiprocessing.pool import ThreadPool


# 定义测试运行成功、失败、错误、以及忽略的测试用例集
cases_run_success = []
cases_run_fail = []
cases_encounter_error = []
skipped_cases = []

# 初始化日志
log = MyLogger(file_name='debug_info.log')


def group_test_cases_by_class(cases_to_run):
    test_groups_dict = OrderedDict()
    for item in cases_to_run:
        tag_filter, cls, func_name, func, value = item
        test_groups_dict.setdefault(cls, []).append((tag_filter, cls, func_name, func, value))
    test_groups = [(x, y) for x, y in zip(test_groups_dict.keys(), test_groups_dict.values())]
    return test_groups


def main(user_options=None):

    start = time.time()
    # 解析用户输入
    customize_option = parse_options(user_options)

    log.debug("日志开始")
    # init global 变量
    init()

    # 设置global 变量
    set_config('config', customize_option.config)

    # 把环境变量写入配置，供测试报告调用
    set_config(get_config('config').setdefault('env', customize_option.default_env), customize_option.default_env)

    # 设置out_put_folder
    out_put_base_folder = customize_option.report_file
    if not out_put_base_folder:
        raise ValueError("Must provide output base folder")
    out_put_folder = out_put_base_folder + os.sep + time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime())

    set_config('out_put_folder', out_put_folder)

    if os.path.exists(out_put_folder):
        shutil.rmtree(out_put_folder)
    os.makedirs(out_put_folder)

    log.debug("传入的自定义变量为{}".format(customize_option))

    # 从默认文件夹tests开始查找测试用例
    case_finder = DiscoverTestCases(customize_option.test_targets)
    # 查找测试模块并导入
    test_module = case_finder.find_test_module()
    # 查找测试用例
    original_test_cases = case_finder.find_tests(test_module)

    # 根据用户输入参数-i进一步筛选
    # 获取到要运行的测试用例，和被忽略的测试用例
    raw_test_suites, excluded_test_suites = TestFilter(original_test_cases).tag_filter_run(customize_option.include_tags_any_match)

    # 获取到最终的测试用例集，并按class名组织
    test_suites = group_test_cases_by_class(raw_test_suites)
    log.info("获取到的测试用例集为{}".format(repr(test_suites)))
    # 获取到被忽略的测试用例集，并按class名组织
    excluded_test_suites = group_test_cases_by_class(excluded_test_suites)
    log.info("忽略运行的测试用例集为{}".format(excluded_test_suites))

    # 直接解析被忽略的测试用例集，输出各项预设信息供测试报告调用
    for s in excluded_test_suites:
        func_tag, func_list = s
        for item in func_list:
            no_run_cls_group, no_run_cls, no_run_cls_name, no_run_func, value = item
            no_run_dict = dict()
            no_run_dict["testcase_id"] = getattr(no_run_cls, 'test_case_id', None)
            no_run_dict["testcase"] = no_run_cls.__name__ + '.' + no_run_func.__name__
            no_run_dict["execution_time"] = None
            no_run_dict["running_testcase_name"] = None
            no_run_dict["class_name"] = no_run_cls.__name__
            no_run_dict['module_name'] = no_run_cls.__name__
            no_run_dict['re_run'] = None
            no_run_dict['log_list'] = None
            # no_run_dict["testcase_filter_tag"] = getattr(no_run_cls, 'tag', None)
            no_run_dict["exception_type"] = None
            no_run_dict["exception_info"] = None
            no_run_dict["screenshot_list"] = []
            skipped_cases.append(no_run_dict)


    # 传入并发数目
    p = ThreadPool(customize_option.test_thread_number)
    # 使用偏函数固定住并发的数目
    # 使用map将test_suites列表中的测试用例一个个传入class_run中运行
    p.map(functools.partial(class_run, test_thread_number=customize_option.test_thread_number), test_suites)
    p.close()
    p.join()

    end = time.time()
    log.info('本次总运行时间 %s s' % (end - start))

    log.info('\nTotal %s cases were run:' % (len(cases_run_success) + len(cases_run_fail) + len(cases_encounter_error)))

    if cases_run_success:
        log.info('\033[1;32m')
        log.info('%s cases Passed, details info are:\n %s' % (
        len(cases_run_success), list(map(lambda x: x["testcase"], cases_run_success))))
        log.info('\033[0m')

    if cases_run_fail:
        log.info('%s cases Failed, details info are:\n %s' % (
        len(cases_run_fail), list(map(lambda x: x["testcase"], cases_run_fail))))
        log.error('\033[1;31m')
    if cases_encounter_error:
        log.info('%s cases Run error, details info are:\n %s' % (
        len(cases_encounter_error), list(map(lambda x: x["testcase"], cases_encounter_error))))
        log.error('\033[0m')

    if skipped_cases:
        log.info('%s cases Skipped, details info are::\n %s' % (
        len(skipped_cases), list(map(lambda x: x["testcase"], skipped_cases))))

    log.info("\n{pre_fix} Tests done {timer}, total used {c_time:.2f} seconds {pre_fix} ".format(pre_fix='---' * 10,
                                                                                                 timer=time.strftime(
                                                                                                     '%Y-%m-%d %H:%M:%S',
                                                                                                     time.localtime(
                                                                                                         end)),
                                                                                                 c_time=end - start))

    # Generate automation report
    log.info("\n{pre_fix} Starting to generate automation report... {pre_fix} ".format(pre_fix='---' * 10))
    generate_html_report(out_put_folder, cases_run_success, cases_run_fail, cases_encounter_error, skipped_cases,
                         time.time(), platform.system(), False)
    log.info("\n{pre_fix} Generate report done, please check report.html from '{report}'. {pre_fix} ".format(
        pre_fix='---' * 10, report=out_put_folder))




def class_run(case, test_thread_number):
    cls, func_pack = case
    log.debug('类 -{}- 开始运行'.format(cls.__name__))
    p = ThreadPool(test_thread_number)
    p.map(func_run, func_pack)
    p.close()
    p.join()
    log.debug('类 -{}- 结束运行'.format(cls.__name__))


# def func_run(test_case):
#     test_tag,  cls, func_name, func, value = test_case
#     if value:
#         func(func, *value)
#     else:
#         func(func)


# used to map all the test cases
def format_time(time_delta: float):
    ms = "{0:0.3f}".format(time_delta % 1000).split('.')[-1]
    seconds = "{0:0>2d}".format(int(time_delta % 60))
    minutes = "{0:0>2d}".format(int(time_delta / 60))
    hours = "{0:0>2d}".format(int(time_delta / (60 * 60)))
    return hours + "h: " + minutes + "m: " + seconds + "s: " + ms + "ms"


def func_run(test_case):
    try:
        f_c = dict()
        # 测试开始时间
        s = time.time()
        test_tag,  cls, func_name, func, value = test_case
        log.debug('类 -{cls}的方法{func}- 开始运行'.format(cls=cls.__name__, func=func_name))

        if value:
            func(func, *value)
        else:
            func(func)

        setattr(func, 'run_status', 'pass')

        # 测试结束时间
        e = time.time()
        log.debug('类 -{cls}的方法{func}- 结束运行'.format(cls=cls.__name__, func=func_name))


        # 超时运行时间，通过get_config获取
        if e - s > get_config('config')["run_time_out"]:
            raise RunTimeTooLong(func_name, e - s)
        f_c["execution_time"] = format_time(e - s)
    except RunTimeTooLong as runtime_err:
        log.error("运行时间过长错误{}".format(repr(runtime_err)))
        setattr(func, 'run_status', 'error')
        setattr(func, 'exception_type', 'RunTimeTooLong')
        setattr(func, 'error_message', repr(traceback.format_exc()))

    except AssertionError as assert_err:
        log.error("断言错误{}".format(repr(assert_err)))

        setattr(func, 'run_status', 'fail')
        setattr(func, 'exception_type', 'AssertionError')
        setattr(func, 'error_message', repr(traceback.format_exc()))
    except Exception as e:
        log.error("Exception错误{}".format(repr(e)))
        setattr(func, 'run_status', 'error')
        setattr(func, 'exception_type', 'Exception')
        setattr(func, 'error_message', repr(traceback.format_exc()))
    finally:
        if not f_c.setdefault("execution_time", None):
            # 当try语句运行出错后，重新获取测试用例结束时间
            e = time.time()
            f_c["execution_time"] = format_time(e - s)
        f_c["testcase"] = cls.__name__ + '.' + func_name
        f_c["testcase_id"] = getattr(cls, 'test_case_id', None)

        f_c["running_testcase_name"] = func_name
        f_c["class_name"] = cls.__name__
        f_c['module_name'] = func.__name__

        f_c["exception_type"] = getattr(func, 'exception_type', None)
        f_c["exception_info"] = getattr(func, 'error_message', None)
        f_c["error_message"] = getattr(func, 'error_message', None)
        f_c["re_run"] = None
        f_c["screenshot_list"] = []

        # 收集测试成功用例
        if getattr(func, 'run_status') == 'pass':
            cases_run_success.append(f_c)
        else:
            # 如果测试运行不成功，则记录日志消息
            func_log_path = os.path.join(get_config('out_put_folder'), func_name)
            if not os.path.exists(func_log_path):
                os.mkdir(func_log_path)

            func_log_file = os.path.join(func_log_path, 'first_run_log.log')
            with open(func_log_file, 'w+', encoding="utf-8") as f:
                f.writelines(repr(getattr(func, 'error_message', None)))
            f_c.setdefault('log_list', []).append('.%s%s%sfirst_run_log.log' % (os.sep, func_name, os.sep))

            # 收集断言错误的测试用例
            if getattr(func, 'run_status') == 'fail':
                cases_run_fail.append(f_c)
            elif getattr(func, "exception_type") == 'RunTimeTooLong':
                # 收集运行时发生错误的测试用例
                cases_encounter_error.append(f_c)


if __name__ == "__main__":
    main()




def init():
    global _config
    _config = {}


def get_config(k):
    try:
        return _config[k]
    except KeyError:
        return None


def set_config(k, v):
    try:
        _config[k] = v
    except KeyError:
        return None



# -*- coding: utf-8 -*-

import time

import os
import traceback

from ztDemo.ztDemo.common.my_logger import MyLogger
from ztDemo.ztDemo.settings.global_config import get_config


my_logger = MyLogger().get_logger(__name__)


# used to map all the test cases
def format_time(time_delta: float):
    ms = "{0:0.3f}".format(time_delta % 1000).split('.')[-1]
    seconds = "{0:0>2d}".format(int(time_delta % 60))
    minutes = "{0:0>2d}".format(int(time_delta / 60))
    hours = "{0:0>2d}".format(int(time_delta / (60 * 60)))
    return hours + "h: " + minutes + "m: " + seconds + "s: " + ms + "ms"


def generate_html_report(output_folder, passed_cases, failed_cases, error_cases, skipped_cases, start_time,
                         platform_info,
                         auto_refresh):
    with open(output_folder + os.sep + "report.html", "w") as report_file_handler:
        back_ground_color_page = "#FFFFFF"
        bg_color_column = "#ADADAD"
        end_time = time.time()
        execution_time = format_time(end_time - start_time)

        css = """
        <style type="text/css">
            a:link,a:visited{
                text-decoration:none;
            }
            a:hover{
                text-decoration:underline;
                background-color:#8E8E8E;
            }
        </style>
        """

        # java script code for filter function
        java_script_code_for_filter = """
        <script type="text/javascript">

            function changeTag(){
                var my_select = document.getElementById("testcase_tag");
                if (my_select.value == "ALL"){
                    //all_failed_test_cases is  represent for all failed test cases
                    if(document.getElementById("all_failed_test_cases")){
                        all_failed_test_case = document.getElementById("all_failed_test_cases");
                        all_failed_test_case.style.display="none";
                        all_failed_test_case.style.display="block";

                        all_failed_case_list = document.getElementsByClassName("all_failed_case");
                        for(var i=0;i<all_failed_case_list.length;i++){
                            all_failed_case_list[i].style.display="block";
                        }
                        document.getElementById("num_fail").innerHTML=i;
                    }

                    if(document.getElementById("all_error_test_cases")){
                            all_error_test_cases = document.getElementById("all_error_test_cases");
                            all_error_test_cases.style.display="none";
                            all_error_test_cases.style.display="block";

                            all_error_case_list = document.getElementsByClassName("all_error_case");
                            for(var i=0;i<all_error_case_list.length;i++){
                                all_error_case_list[i].style.display="block";
                            }
                            document.getElementById("num_error").innerHTML=i;
                        }

                    if(document.getElementById("all_skipped_test_cases")){
                            all_skipped_test_cases = document.getElementById("all_skipped_test_cases");
                            all_skipped_test_cases.style.display="none";
                            all_skipped_test_cases.style.display="block";

                            all_skipped_case_list = document.getElementsByClassName("all_skipped_case");
                            for(var i=0;i<all_skipped_case_list.length;i++){
                                all_skipped_case_list[i].style.display="block";
                            }
                            document.getElementById("num_skip").innerHTML=i;
                        }   

                    //all_passed_test_cases is  represent for all passed test cases
                    if (document.getElementById("all_passed_test_cases")){
                        all_passed_test_case = document.getElementById("all_passed_test_cases");
                        all_passed_test_case.style.display="none";
                        all_passed_test_case.style.display="block";
                        all_passed_case_list = document.getElementsByClassName("all_passed_case");

                        for(var j=0;j<all_passed_case_list.length;j++){
                            all_passed_case_list[j].style.display="block";
                        }
                        document.getElementById("num_pass").innerHTML=j;
                    }
                }

                var l = new Array()
                k = 0;
                all_failed_case_list = document.getElementsByClassName("all_failed_case");

                for(var i=0;i<all_failed_case_list.length;i++){
                    all_failed_case_list[i].style.display="none";

                    if (all_failed_case_list[i].getAttribute("name").indexOf(my_select.value) != -1){
                        l[k] = all_failed_case_list[i];
                        k = k + 1;
                    }
                }

                all_error_case_list = document.getElementsByClassName("all_error_case");

                for(var i=0;i<all_error_case_list.length;i++){
                    all_error_case_list[i].style.display="none";

                    if (all_error_case_list[i].getAttribute("name").indexOf(my_select.value) != -1){
                        l[k] = all_error_case_list[i];
                        k = k + 1;
                    }
                }

                all_passed_case_list = document.getElementsByClassName("all_passed_case");

                for(var i=0;i<all_passed_case_list.length;i++){
                    all_passed_case_list[i].style.display="none";
                    if (all_passed_case_list[i].getAttribute("name").indexOf(my_select.value) != -1){
                        l[k] = all_passed_case_list[i];
                        k = k + 1;
                    }
                }

                all_skipped_case_list = document.getElementsByClassName("all_skipped_case");

                for(var i=0;i<all_skipped_case_list.length;i++){
                    all_skipped_case_list[i].style.display="none";
                    if (all_skipped_case_list[i].getAttribute("name").indexOf(my_select.value) != -1){
                        l[k] = all_skipped_case_list[i];
                        k = k + 1;
                    }
                }

                if (document.getElementById("all_failed_test_cases")){
                    all_failed_test_case = document.getElementById("all_failed_test_cases");
                    all_failed_test_case.style.display="none";
                    all_failed_test_case.style.display="block";
                }

                if (document.getElementById("all_error_test_cases")){
                    all_error_test_case = document.getElementById("all_error_test_cases");
                    all_error_test_case.style.display="none";
                    all_error_test_case.style.display="block";
                }

                if (document.getElementById("all_passed_test_cases")){
                    all_passed_test_case = document.getElementById("all_passed_test_cases");
                    all_passed_test_case.style.display="none";
                    all_passed_test_case.style.display="block";
                }

                if (document.getElementById("all_skipped_test_cases")){
                    all_skipped_test_case = document.getElementById("all_skipped_test_cases");
                    all_skipped_test_case.style.display="none";
                    all_skipped_test_case.style.display="block";
                }
                num_failed_case = 0;
                num_error_case = 0;
                num_passed_case = 0;
                num_skipped_case = 0;

                for(var i=0;i<l.length;i++){
                    l[i].style.display="block";
                    if ("all_failed_case"==l[i].className){
                        num_failed_case = num_failed_case + 1;
                    }
                    if ("all_error_case"==l[i].className){
                        num_error_case = num_error_case + 1;
                    }
                    if ("all_passed_case"==l[i].className){
                        num_passed_case = num_passed_case + 1;
                    }
                    if ("all_skipped_case"==l[i].className){
                        num_skipped_case = num_skipped_case + 1;
                    }
                }

                if (document.getElementById("num_fail")){
                    document.getElementById("num_fail").innerHTML=num_failed_case;
                }
                if (document.getElementById("num_error")){
                    document.getElementById("num_error").innerHTML=num_error_case;
                }
                if (document.getElementById("num_pass")){
                    document.getElementById("num_pass").innerHTML=num_passed_case;
                }
                if (document.getElementById("num_skip")){
                    document.getElementById("num_skip").innerHTML=num_skipped_case;
                }
            }

        </script>
        """

        java_script_copy_failed_testcases = """
        <script>
            function copyFailedTestCase(){
                document.getElementById("failed_testcase_names").style.display="none";
                if (%s === 1){
                    document.getElementById("copy_status_info").innerHTML = "<font color=red>Copied 1 TestCase</font>";
                } else if(%s > 1){
                    document.getElementById("copy_status_info").innerHTML = "<font color=red>Copied %s TestCases</font>";
                }
                setTimeout(function(){
                    document.getElementById("copy_status_info").innerHTML = "";
                    document.getElementById("failed_testcase_names").style.display="block";
                }, 2000);
            }
        </script>
        """ % (len(failed_cases), len(failed_cases), len(failed_cases))

        refresh_string = "<meta http-equiv=\"Refresh\" content=\"25\" />"

        str_high_chart = """
        <html>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />

            <head>%s
                <title>Automation Test Report</title>
                %s
                %s
                %s
            </head>
            <script type="text/javascript" ztDemo="http://cdn.hcharts.cn/jquery/jquery-1.8.3.min.js"></script>
            <script type="text/javascript" ztDemo="http://cdn.hcharts.cn/highcharts/highcharts.js"></script>
            <script type="text/javascript" ztDemo="http://cdn.hcharts.cn/highcharts/highcharts-3d.js"></script>
            <script type="text/javascript" ztDemo="https://cdn.jsdelivr.net/clipboard.js/1.5.12/clipboard.min.js"></script>
            <script>
                $(function () {
                    $('#chart_div').highcharts({
                        chart: {
                            type: 'pie',
                            options3d: {
                                enabled: true,
                                alpha: 50,
                                beta: 0
                            }
                        },
                        credits: {
                            enabled: false
                        },
                        colors:[
                            '#478608',
                            '#760202',
                            '#FF0303'
                          ],
                        title: {
                            text: 'Automation Test Report'
                        },
                        tooltip: {
                            pointFormat: '{series.name}: <b>{point.percentage:.1f}%%</b>'
                        },
                        plotOptions: {
                            pie: {
                                size:'100%%',
                                allowPointSelect: true,
                                cursor: 'pointer',
                                depth: 30,
                                dataLabels: {
                                    enabled: true,
                                    formatter: function(){
                                        return this.point.name+"-"+this.percentage.toFixed(1)+"%%";
                                    }
                                }
                            }
                        },
                        series: [{
                            type: 'pie',
                            name: 'Testcase Result',
                            data: [
                                ['Passed',   %s],
                                ['Failed',   %s],
                                ['RunError', %s]
                            ]
                        }]
                    });
                });
              </script>
        """ % (
            refresh_string if auto_refresh else "",
            css,
            java_script_code_for_filter,
            java_script_copy_failed_testcases,
            len(passed_cases),
            len(failed_cases),
            len(error_cases)
        )

        report_file_handler.write(str_high_chart)
        # 从全局变量中获取environment
        environment = get_config('config')["env"]

        str_top_table = """
        <body  bgcolor='%s'>
        <table align='center' cellpadding=0 cellspacing=0 style='border-collapse: collapse' width='925'>
            <tr>
                <td>
                    <div id='chart_div' style="min-width: 500px; height: 300px; margin: 0 auto"></div>
                </td>
                <td>
                    <table border='1' align='center' cellpadding=0 cellspacing=0 style='border-collapse: collapse' width='400'>
                        <tr  bgcolor='%s'>
                            <td>
                                <font color='blue'>Report Generate Time</font>
                            </td>
                            <td width='150' align='center'>
                                <font color='blue'>%s</font>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <font color='blue'>Total Execution Time</font>
                            </td>
                            <td width='150' align='center'><font color='blue'>%s</font>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <font color='blue'>Total TestCases</font>
                            </td>
                            <td align='center' width='150'>
                                <font color='blue'>%s</font>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <font color='blue'>TestCases Passed</font>
                            </td>
                            <td align='center'>
                                <font color='blue'>%s</font>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <font color='blue'>TestCases Failed</font>
                            </td>
                            <td align='center'>
                                <font color='blue'>%s</font>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <font color='blue'>TestCases Run Error</font>
                            </td>
                            <td align='center'>
                                <font color='blue'>%s</font>
                            </td>
                        </tr>

                        <tr>
                            <td>
                                <font color='blue'>Environment</font>
                            </td>
                            <td align='center'>
                                <font color='blue'>%s</font>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <font color='blue'>Platform</font>
                            </td>
                            <td align='center'>
                                <font color='blue'>%s</font>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <font color='blue'>AutomationFramework</font>
                            </td>
                            <td align='center'>
                            <font color='blue'>请关注微信公众号 - iTesting</font>
                            </td>
                        </tr>
                    </table>
                </td>
            </tr>
        </table>
        <br>
        """ % (
            back_ground_color_page,
            bg_color_column,
            time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time())),
            execution_time,
            len(passed_cases) + len(failed_cases),
            len(passed_cases),
            len(failed_cases),
            len(error_cases),
            environment,
            platform_info
        )

        report_file_handler.write(str_top_table)
        all_testcases = passed_cases + failed_cases + error_cases + skipped_cases

        # generate the drop down list for all the different tags
        html_drop_down_list = """
        <table border='0' align='center' cellpadding=0 cellspacing=0 style='border-collapse: collapse' width='925'>
            <tr>
                <td width='200'>Filter TestCases By Package: </td>
                <td>
        """
        report_file_handler.write(html_drop_down_list)

        all_testcases_package_list = passed_cases + failed_cases + error_cases + skipped_cases

        report_file_handler.write("<select id=\"testcase_tag\" onchange=\"changeTag()\">")
        filter_class = []
        for testcase in all_testcases_package_list:
            filter_class.append('ALL' + '.' + testcase["class_name"])

        report_file_handler.write("<option value='ALL'>")
        report_file_handler.write("ALL")
        report_file_handler.write("</option>")
        for item in set(filter_class):
            report_file_handler.write("<option value='%s'>" % item)
            report_file_handler.write("%s" % item[4:])
            report_file_handler.write("</option>")
        report_file_handler.write("</select>")
        report_file_handler.write("</td><td align='left'><p id='copy_status_info'></p></td><td>")

        if len(failed_cases) > 0:
            str_failed_testcase_names = ""

            for failed_testcase in failed_cases:
                str_failed_testcase_names += failed_testcase["testcase"] + " "

            str_failed_testcases = """
            <div align="center">
                <button id="failed_testcase_names" class='failed_testcase_names' data-clipboard-text="%s" onclick="copyFailedTestCase()">
                    Click Me to Copy the Failed TestCase Names to Clipboard!
                </button>
            </div>
            <script>
                new Clipboard('.failed_testcase_names');
            </script>
            """ % str_failed_testcase_names

            report_file_handler.write(str_failed_testcases)

        report_file_handler.write("</td></tr></table>")
        report_file_handler.write("<br>")

        if len(failed_cases) > 0:
            failed_testcase_table = """
            <table border='1' align='center' cellpadding=0 cellspacing=0 style='border-collapse: collapse' width='925'>
                <tr id='all_failed_test_cases' bgcolor='%s' width='925'>
                    <td width='125' align='center'>
                        <b>
                            <font color='red'>TestCaseID</font>
                        </b>
                    </td>
                    <td width='650' style='word-break:break-all' align='center'>
                        <b>
                            <font id='num_fail' color='red'>%s Test %s Failed</font>
                        </b>
                    </td>
                    <td width='150' align='center'>
                        <b>
                            <font color='red'>Execution Time</font>
                        </b>
                    </td>
                </tr>
            """ % (
                bg_color_column,
                len(failed_cases),
                "Cases" if len(failed_cases) > 1 else "Case"
            )

            report_file_handler.write(failed_testcase_table)

            for failed_testcase in failed_cases:
                module_name = failed_testcase["running_testcase_name"]
                module_name_display = failed_testcase["testcase"]
                report_file_handler.write(
                    "<tr class='all_failed_case' name=ALL.%s  width='925'>" % failed_testcase["testcase"])

                report_file_handler.write(
                    "<td width='125' align='center' style='word-break:break-all;no-wrap:no-wrap'>")
                report_file_handler.write(
                    "<a title='Click to go to test center' href='https://testcenter.englishtown.com/testlink/linkto.php?tprojectPrefix=ET&item=testcase&id=ET-%s'><font color='red'>" %
                    failed_testcase["testcase_id"])
                report_file_handler.write("%s" % failed_testcase["testcase_id"])
                report_file_handler.write("</font></a>")
                report_file_handler.write("</td>")

                testcase_information = """
                    <td width='650'style='word-break:break-all'>
                    <a  href='%s'>
                        <font color='blue'>
                            %s
                        </font>
                    </a>
                    """ % (
                    module_name,
                    module_name_display
                )

                report_file_handler.write(testcase_information)

                report_file_handler.write(
                    " <a title='%s'> (" % 'Please see the execution logs for more details' + failed_testcase[
                        "exception_type"] + ") </a>")

                try:
                    report_file_handler.write(
                        "<a title='Log' href='%s'>        <font color='#F18428'>   Log</font> </a>" %
                        list(failed_testcase["log_list"])[0])
                except BaseException:
                    my_logger.error(traceback.format_exc())

                try:
                    if failed_testcase["screenshot_list"]:
                        report_file_handler.write(
                            "<a title='Screenshot' href='%s'> <font color='#F18428'> - Screenshot</font> </a>" %
                            list(failed_testcase["screenshot_list"])[0])
                except BaseException:
                    my_logger.error(traceback.format_exc())

                try:
                    if len(list(failed_testcase["log_list"])) > 1:
                        report_file_handler.write(
                            "<a title='ReRun_Log' href='%s'> <font color='#F18428'> | ReRun_Log</font> </a>" %
                            list(failed_testcase["log_list"])[1])
                except BaseException:
                    my_logger.error(traceback.format_exc())

                try:

                    if len(list(failed_testcase["screenshot_list"])) > 1:
                        report_file_handler.write(
                            "<a title='ReRun_Screenshot' href='%s'> <font color='#F18428'> - ReRun_Screenshot</font> </a>" %
                            list(failed_testcase["screenshot_list"])[1])
                except BaseException:
                    my_logger.error(traceback.format_exc())

                report_file_handler.write("</td>")

                report_file_handler.write("<td width='150' align='center' >")
                report_file_handler.write("<font color='red'>")
                report_file_handler.write("%s" % failed_testcase["execution_time"])
                report_file_handler.write("</font>")
                report_file_handler.write("</td></tr>")
            report_file_handler.write("</table>")
        report_file_handler.write("<br>")

        if len(error_cases) > 0:
            error_testcase_table = """
            <table border='1' align='center' cellpadding=0 cellspacing=0 style='border-collapse: collapse' width='925'>
                <tr id='all_error_test_cases' bgcolor='%s' width='925'>
                    <td width='125' align='center'>
                        <b>
                            <font color='red'>TestCaseID</font>
                        </b>
                    </td>
                    <td width='650' style='word-break:break-all' align='center'>
                        <b>
                            <font id='num_error' color='red'>%s Test %s encountered error during run</font>
                        </b>
                    </td>
                    <td width='150' align='center'>
                        <b>
                            <font color='red'>Execution Time</font>
                        </b>
                    </td>
                </tr>
            """ % (
                bg_color_column,
                len(error_cases),
                "Cases" if len(list(error_cases)) > 1 else "Case"
            )

            report_file_handler.write(error_testcase_table)

            for error_testcase in error_cases:
                module_name = error_testcase["running_testcase_name"]
                module_name_display = error_testcase["testcase"]
                report_file_handler.write(
                    "<tr class='all_error_case' name=ALL.%s  width='925'>" % error_testcase["testcase"])

                report_file_handler.write(
                    "<td width='125' align='center' style='word-break:break-all;no-wrap:no-wrap'>")
                report_file_handler.write(
                    "<a title='Click to go to test center' href='https://testcenter.xxxx.com/testlink/linkto.php?tprojectPrefix=ET&item=testcase&id=ET-%s'><font color='red'>" %
                    error_testcase["testcase_id"])
                report_file_handler.write("%s" % error_testcase["testcase_id"])
                report_file_handler.write("</font></a>")
                report_file_handler.write("</td>")

                testcase_information = """
                    <td width='650'style='word-break:break-all'>
                    <a  href='%s'>
                        <font color='blue'>
                            %s
                        </font>
                    </a>
                    """ % (
                    module_name,
                    module_name_display
                )

                report_file_handler.write(testcase_information)

                report_file_handler.write(
                    " <a title='%s'> (" % 'Please see the execution logs for more details' + error_testcase[
                        "exception_type"] + ") </a>")

                try:
                    report_file_handler.write(
                        "<a title='Log' href='%s'>        <font color='#F18428'>   Log</font> </a>" %
                        list(error_testcase["log_list"])[0])
                except BaseException:
                    my_logger.error(traceback.format_exc())

                try:
                    if error_testcase["screenshot_list"]:
                        report_file_handler.write(
                            "<a title='Screenshot' href='%s'> <font color='#F18428'> - Screenshot</font> </a>" %
                            list(error_testcase["screenshot_list"])[0])
                except BaseException:
                    my_logger.error(traceback.format_exc())

                try:
                    if len(list(error_testcase["log_list"])) > 1:
                        report_file_handler.write(
                            "<a title='ReRun_Log' href='%s'> <font color='#F18428'> | ReRun_Log</font> </a>" %
                            list(error_testcase["log_list"])[1])
                except BaseException:
                    my_logger.error(traceback.format_exc())

                try:

                    if len(list(error_testcase["screenshot_list"])) > 1:
                        report_file_handler.write(
                            "<a title='ReRun_Screenshot' href='%s'> <font color='#F18428'> - ReRun_Screenshot</font> </a>" %
                            list(error_testcase["screenshot_list"])[1])
                except BaseException:
                    my_logger.error(traceback.format_exc())

                report_file_handler.write("</td>")

                report_file_handler.write("<td width='150' align='center' >")
                report_file_handler.write("<font color='red'>")
                report_file_handler.write("%s" % error_testcase["execution_time"])
                report_file_handler.write("</font>")
                report_file_handler.write("</td></tr>")
            report_file_handler.write("</table>")
        report_file_handler.write("<br>")

        if len(passed_cases) > 0:
            passed_testcase_table = """
            <table border='1' align='center' cellpadding=0 cellspacing=0 style='border-collapse: collapse' width='925'>
                <tr id='all_passed_test_cases' bgcolor='%s' width='925'>
                    <td width='125' align='center'>
                        <b>
                            <font color='blue'>TestCaseID</font>
                        </b>
                    </td>
                    <td width='650' style='word-break:break-all' align='center'>
                        <b>
                            <font id='num_pass' color='blue'>%s  Test %s Passed</font>
                        </b>
                    </td>
                    <td width='150' align='center'>
                        <b>
                            <font color='blue'>Execution Time</font>
                        </b>
                    </td>
                </tr>
            """ % (
                bg_color_column,
                len(passed_cases),
                "Cases" if len(list(passed_cases)) > 1 else "Case"
            )
            report_file_handler.write(passed_testcase_table)

            for passed_testcase in passed_cases:
                module_name = passed_testcase["running_testcase_name"]
                module_name_display = passed_testcase['testcase']

                report_file_handler.write(
                    "<tr class='all_passed_case' name=ALL.%s  width='925'>" % passed_testcase["testcase"])

                report_file_handler.write("<td width='125' align='center' >")
                report_file_handler.write(
                    "<a title='Click to go to test center' href='https://testcenter.xxxx.com/testlink/linkto.php?tprojectPrefix=ET&item=testcase&id=ET-%s'> <font color='blue'>" %
                    passed_testcase["testcase_id"])
                report_file_handler.write("%s" % passed_testcase["testcase_id"])
                report_file_handler.write("</font></a>")
                report_file_handler.write("</td>")

                report_file_handler.write("<td width='650' style='word-break:break-all'>")
                report_file_handler.write("<a title='Click to see the log & screenshot' href='")
                report_file_handler.write(module_name)
                report_file_handler.write(
                    "'><font color='%s'>" % (
                        "blue" if passed_testcase["re_run"] == 0 else "green"))
                report_file_handler.write(module_name_display)

                # show re-run got passed in report
                if passed_testcase["re_run"] == 1:
                    report_file_handler.write("<font color='chocolate'> -- Re-Run Passed</font>")

                if passed_testcase in error_cases:
                    report_file_handler.write("<font color='orange'> -- Miss Attribute</font>")

                report_file_handler.write("</font>")
                report_file_handler.write("</font></a>")

                report_file_handler.write("</td>")

                report_file_handler.write("<td width='150' align='center'>")
                report_file_handler.write("<font color='blue'>")

                report_file_handler.write("%s" % passed_testcase["execution_time"])
                report_file_handler.write("</font>")
                report_file_handler.write("</td></tr>")
            report_file_handler.write("</table>")
        report_file_handler.write("<br>")

        if len(skipped_cases) > 0:
            skipped_testcase_table = """
            <table border='1' align='center' cellpadding=0 cellspacing=0 style='border-collapse: collapse' width='925'>
                <tr id='all_skipped_test_cases' bgcolor='%s' width='925'>
                    <td width='125' align='center'>
                        <b>
                            <font color='gray'>TestCaseID</font>
                        </b>
                    </td>
                    <td width='650' style='word-break:break-all' align='center'>
                        <b>
                            <font id='num_skip' color='gray'>%s  Test %s Skipped</font>
                        </b>
                    </td>
                    <td width='150' align='center'>
                        <b>
                            <font color='gray'>Execution Time</font>
                        </b>
                    </td>
                </tr>
            """ % (
                bg_color_column,
                len(skipped_cases),
                "Cases" if len(list(skipped_cases)) > 1 else "Case"
            )
            report_file_handler.write(skipped_testcase_table)

            for skipped_testcase in skipped_cases:
                module_name = skipped_testcase["running_testcase_name"]
                module_name_display = skipped_testcase['testcase']

                report_file_handler.write(
                    "<tr class='all_skipped_case' name=ALL.%s  width='925'>" % skipped_testcase["testcase"])

                report_file_handler.write("<td width='125' align='center' >")
                report_file_handler.write(
                    "<a title='Click to go to test center' href='https://testcenter.xxxx.com/testlink/linkto.php?tprojectPrefix=ET&item=testcase&id=ET-%s'> <font color='gray'>" %
                    skipped_testcase["testcase_id"])
                report_file_handler.write("%s" % skipped_testcase["testcase_id"])
                report_file_handler.write("</font></a>")
                report_file_handler.write("</td>")

                report_file_handler.write("<td width='650' style='word-break:break-all'>")
                report_file_handler.write("<a title='Click to see the log & screenshot' href='")
                report_file_handler.write(module_name if module_name else "None")
                report_file_handler.write(
                    "'><font color='%s'>" % (
                        "gray"))
                report_file_handler.write(module_name_display)

                # show re-run got passed in report
                if skipped_testcase["re_run"] == 1:
                    report_file_handler.write("<font color='chocolate'> -- Re-Run Passed</font>")

                if skipped_testcase in error_cases:
                    report_file_handler.write("<font color='orange'> -- Miss Attribute</font>")

                report_file_handler.write("</font>")
                report_file_handler.write("</font></a>")

                report_file_handler.write("</td>")

                report_file_handler.write("<td width='150' align='center'>")
                report_file_handler.write("<font color='gray'>")

                report_file_handler.write("%s" % skipped_testcase["execution_time"])
                report_file_handler.write("</font>")
                report_file_handler.write("</td></tr>")
            report_file_handler.write("</table>")
        report_file_handler.write("<br>")

        report_file_handler.write("<br><br>")
        report_file_handler.write("</body></html>")



# -*- coding: utf-8 -*-
import re

import yaml
import os

from openpyxl import load_workbook, Workbook
from openpyxl.cell import MergedCell


def data_provider(test_data):
    def wrapper(func):
        setattr(func, "__data_Provider__", test_data)
        global index_len
        index_len = len(str(len(test_data)))
        return func
    return wrapper



def mk_test_name(name, value, index=0):
    index = "{0:0{1}d}".format(index+1, index_len)
    test_name = "{0}_{1}_{2}".format(name, index, str(value))
    return re.sub(r'\W|^(?=\d)', '_', test_name)


class ExcelDataProcessor:
    def __init__(self, file_path):
        self.file_path = file_path

        if not file_path.endswith('.xlsx'):
            raise TypeError("不是一个有效的EXCEL文件")

        if not os.path.exists(self.file_path):
            Workbook().save(self.file_path)
            print("源文件不存在，在路径{}新建它".format(self.file_path))
        else:
            print("源文件{}存在".format(self.file_path))

    def read_data(self, sheet_name):
        all_data = []
        wb = load_workbook(self.file_path)
        if sheet_name not in wb.sheetnames:
            raise NameError('''Excel Tab "{}" 在文件 "{}" 中不存在'''.format(sheet_name, self.file_path))
        sheet = wb[sheet_name]
        print(f"Title = {sheet.title}")
        for row in sheet.rows:
            for cell in row:
                if isinstance(cell, MergedCell):
                    # 如果是Merged Cell就跳过
                    continue
                all_data.append(f"{cell.column_letter}{cell.row} = {cell.value}")
        return all_data

    def write_data(self, sheet_name, column, value):
        wb = load_workbook(self.file_path)
        wb.active
        if sheet_name not in wb.sheetnames:
            raise NameError('''Excel Tab "{}" 在文件 "{}" 中不存在'''.format(sheet_name, self.file_path))
        try:
            wb[sheet_name][column] = value
            wb.save(self.file_path)
        except:
            raise Exception("写入失败")



class YamlDataProcessor:
    def __init__(self, file_path):
        self.file_path = file_path

        if not file_path.endswith('.yaml'):
            raise TypeError("不是一个有效的YAML文件")

        if not os.path.exists(self.file_path):
            with open(self.file_path, 'w'):
                print("源文件不存在，在路径{}新建它".format(self.file_path))
        else:
            print("源文件{}存在".format(self.file_path))

    def read_yaml(self):
        with open(self.file_path, 'r') as stream:
            try:
                return yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                print(exc)